@@ -8,7 +8,7 @@ no warnings 'recursion';
 
 package XML::Compile::Schema::BuiltInFacets;
 use vars '$VERSION';
-$VERSION = '1.39';
+$VERSION = '1.40';
 
 use base 'Exporter';
 
@@ -62,6 +62,8 @@ my %facets_date =  # inclusive or exclusive times is rather useless.
   , minInclusive    => \&_d_min
   , enumeration     => \&_enumeration
   , pattern         => \&_pattern
+  , whiteSpace      => \&_d_whitespace
+  , explicitTimeZone=> \&_d_expl_tz
   );
 
 sub builtin_facet($$$$$$$$)
@@ -96,6 +98,16 @@ sub _s_whiteSpace($$$)
           , ws => $ws, path => $path;
 }
 
+sub _d_whiteSpace($$$)
+{   my ($path, undef, $ws) = @_;
+
+    # whitespace processing already in the dateTime parser
+    $ws eq 'collapse'
+        or error __x"illegal whiteSpace facet '{ws}' in {path}"
+          , ws => $ws, path => $path;
+    ();
+}
+
 sub _whitespace_replace($)
 {   (my $value = shift) =~ s/[\t\r\n]/ /gs;
     $value;
@@ -153,8 +165,8 @@ sub _s_minExclusive($$$)
 {   my ($path, $args, $min) = @_;
     $min = _maybe_big $path, $args, $min;
     sub { return $_[0] if $_[0] > $min;
-        error __x"too small exclusive {value}, larger {min} at {where}"
-          , value => $_[0], min => $min, where => $path;
+          error __x"too small exclusive {value}, larger {min} at {where}"
+             , value => $_[0], min => $min, where => $path;
     };
 }
 
@@ -400,4 +412,23 @@ sub _d_max($$$)
     };
 }
 
+sub _d_expl_tz($$$)
+{   my ($path, $args, $enum) = @_;
+    my $tz = qr/Z$ | [+-](?:(?:0[0-9]|1[0-3])\:[0-5][0-9] | 14\:00)$/x;
+
+      $enum eq 'optional'   ? ()
+    : $enum eq 'prohibited'
+    ? sub {   $_[0] !~ $tz
+                 or error __x"timezone forbidden on {date} at {where}"
+                    , date => $_[0], where => $path;
+          }
+    : $enum eq 'required'
+    ? sub {   $_[0] =~ $tz
+                 or error __x"timezone required on {date} at {where}"
+                    , date => $_[0], where => $path;
+          }
+    : error __x"illegal explicitTimeZone facet '{enum}' in {path}"
+          , enum => $enum, path => $path;
+}
+
 1;
